@@ -17,19 +17,12 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-import sys
-import os
-import logging
 import datetime
+import logging
+import os
 
-import gtk
-import gobject
-
-if __name__ == '__main__':
-    sys.path.insert(0, os.path.abspath("./../../"))
-    logging.basicConfig(level=logging.DEBUG)
-    from rednotebook.journal import Journal
-
+from gi.repository import GObject
+from gi.repository import Gtk
 
 from rednotebook.util import filesystem
 from rednotebook.util import markup
@@ -47,40 +40,41 @@ class DatePage(AssistantPage):
 
         self.journal = journal
 
-        self.all_days_button = gtk.RadioButton(label=_('Export all days'))
-        self.selected_text_button = gtk.RadioButton(
+        self.all_days_button = Gtk.RadioButton(label=_('Export all days'))
+        self.selected_text_button = Gtk.RadioButton(
             group=self.all_days_button)
-        self.one_day_button = gtk.RadioButton(
+        self.one_day_button = Gtk.RadioButton(
             label=_('Export currently visible day'),
             group=self.all_days_button)
-        self.sel_days_button = gtk.RadioButton(
+        self.sel_days_button = Gtk.RadioButton(
             label=_('Export days in the selected time range'),
             group=self.all_days_button)
 
-        self.pack_start(self.all_days_button, False)
-        self.pack_start(self.one_day_button, False)
-        self.pack_start(self.selected_text_button, False)
-        self.pack_start(self.sel_days_button, False)
+        self.pack_start(self.all_days_button, False, False, 0)
+        self.pack_start(self.one_day_button, False, False, 0)
+        self.pack_start(self.selected_text_button, False, False, 0)
+        self.pack_start(self.sel_days_button, False, False, 0)
 
-        label1 = gtk.Label()
+        label1 = Gtk.Label()
         label1.set_markup('<b>' + _('From:') + '</b>')
-        label2 = gtk.Label()
+        label2 = Gtk.Label()
         label2.set_markup('<b>' + _('To:') + '</b>')
 
-        self.calendar1 = Calendar()
-        self.calendar2 = Calendar()
+        show_week_numbers = self.journal.config.read('weekNumbers')
+        self.calendar1 = Calendar(week_numbers=show_week_numbers)
+        self.calendar2 = Calendar(week_numbers=show_week_numbers)
 
-        vbox1 = gtk.VBox()
-        vbox2 = gtk.VBox()
-        vbox1.pack_start(label1, False)
-        vbox1.pack_start(self.calendar1)
-        vbox2.pack_start(label2, False)
-        vbox2.pack_start(self.calendar2)
+        vbox1 = Gtk.VBox()
+        vbox2 = Gtk.VBox()
+        vbox1.pack_start(label1, False, False, 0)
+        vbox1.pack_start(self.calendar1, True, True, 0)
+        vbox2.pack_start(label2, False, False, 0)
+        vbox2.pack_start(self.calendar2, True, True, 0)
 
-        hbox = gtk.HBox()
-        hbox.pack_start(vbox1)
-        hbox.pack_start(vbox2)
-        self.pack_start(hbox)
+        hbox = Gtk.HBox()
+        hbox.pack_start(vbox1, True, True, 0)
+        hbox.pack_start(vbox2, True, True, 0)
+        self.pack_start(hbox, True, True, 0)
 
         self.sel_days_button.connect('toggled', self._on_select_days_toggled)
 
@@ -119,7 +113,8 @@ class DatePage(AssistantPage):
         if enable_selected_text_button:
             self.selected_text_button.set_label(selected_text_label)
         else:
-            self.selected_text_button.set_label(selected_text_label + ' ' +
+            self.selected_text_button.set_label(
+                selected_text_label + ' ' +
                 _('(Only available when text is selected in edit mode)'))
             if self.selected_text_button.get_active():
                 self.selected_text_button.set_active(False)
@@ -140,68 +135,68 @@ class ContentsPage(AssistantPage):
         self.date_format = options.DateFormatOption(_('Date format'), 'exportDateFormat')
         self.date_format.combo.combo_box.set_tooltip_text(_('Leave blank to omit dates in export'))
 
-        self.text_and_tags_button = gtk.RadioButton(label=_('Export text and tags'))
-        self.text_only_button = gtk.RadioButton(label=_('Export text only'),
+        self.text_and_tags_button = Gtk.RadioButton(label=_('Export text and tags'))
+        self.text_only_button = Gtk.RadioButton(label=_('Export text only'),
                                                 group=self.text_and_tags_button)
-        self.tags_only_button = gtk.RadioButton(label=_('Export tags only'),
+        self.tags_only_button = Gtk.RadioButton(label=_('Export tags only'),
                                                 group=self.text_and_tags_button)
-        self.filter_tags_button = gtk.CheckButton(label=_('Filter days by tags'))
+        self.filter_tags_button = Gtk.CheckButton(label=_('Filter days by tags'))
 
-        self.pack_start(self.date_format, False)
-        self.pack_start(self.text_and_tags_button, False)
-        self.pack_start(self.text_only_button, False)
-        self.pack_start(self.tags_only_button, False)
-        self.pack_start(gtk.HSeparator(), False)
-        self.pack_start(self.filter_tags_button, False)
+        self.pack_start(self.date_format, False, False, 0)
+        self.pack_start(self.text_and_tags_button, False, False, 0)
+        self.pack_start(self.text_only_button, False, False, 0)
+        self.pack_start(self.tags_only_button, False, False, 0)
+        self.pack_start(Gtk.HSeparator(), False, False, 0)
+        self.pack_start(self.filter_tags_button, False, False, 0)
 
         self.available_categories = customwidgets.CustomListView([(_('Available tags'), str)])
         self.selected_categories = customwidgets.CustomListView([(_('Selected tags'), str)])
 
-        left_scroll = gtk.ScrolledWindow()
+        left_scroll = Gtk.ScrolledWindow()
         left_scroll.add(self.available_categories)
 
-        right_scroll = gtk.ScrolledWindow()
+        right_scroll = Gtk.ScrolledWindow()
         right_scroll.add(self.selected_categories)
 
-        self.select_button = gtk.Button(_('Select') + ' >>')
-        self.deselect_button = gtk.Button('<< ' + _('Deselect'))
+        self.select_button = Gtk.Button(_('Select') + ' >>')
+        self.deselect_button = Gtk.Button('<< ' + _('Deselect'))
 
         self.select_button.connect('clicked', self.on_select_category)
         self.deselect_button.connect('clicked', self.on_deselect_category)
 
-        centered_vbox = gtk.VBox()
-        centered_vbox.pack_start(self.select_button, True, False)
-        centered_vbox.pack_start(self.deselect_button, True, False)
+        centered_vbox = Gtk.VBox()
+        centered_vbox.pack_start(self.select_button, True, False, 0)
+        centered_vbox.pack_start(self.deselect_button, True, False, 0)
 
-        vbox = gtk.VBox()
-        vbox.pack_start(centered_vbox, True, False)
+        vbox = Gtk.VBox()
+        vbox.pack_start(centered_vbox, True, False, 0)
 
-        hbox = gtk.HBox()
-        hbox.pack_start(left_scroll)
-        hbox.pack_start(vbox, False)
-        hbox.pack_start(right_scroll)
-        self.pack_start(hbox)
+        hbox = Gtk.HBox()
+        hbox.pack_start(left_scroll, True, True, 0)
+        hbox.pack_start(vbox, False, False, 0)
+        hbox.pack_start(right_scroll, True, True, 0)
+        self.pack_start(hbox, True, True, 0)
 
-        self.error_text = gtk.Label('')
+        self.error_text = Gtk.Label(label='')
         self.error_text.set_alignment(0.0, 0.5)
 
-        self.pack_end(self.error_text, False, False)
+        self.pack_end(self.error_text, False, False, 0)
 
         self.text_and_tags_button.set_active(True)
         self.filter_tags_button.connect('toggled', self.check_selection)
 
     def refresh_categories_list(self):
-        model_available = gtk.ListStore(gobject.TYPE_STRING)
+        model_available = Gtk.ListStore(GObject.TYPE_STRING)
         for category in self.journal.categories:
             model_available.append([category])
         self.available_categories.set_model(model_available)
 
-        model_selected = gtk.ListStore(gobject.TYPE_STRING)
+        model_selected = Gtk.ListStore(GObject.TYPE_STRING)
         self.selected_categories.set_model(model_selected)
 
     def on_select_category(self, widget):
         selection = self.available_categories.get_selection()
-        nb_selected, selected_iter = selection.get_selected()
+        _, selected_iter = selection.get_selected()
 
         if selected_iter is not None:
             model_available = self.available_categories.get_model()
@@ -218,7 +213,7 @@ class ContentsPage(AssistantPage):
 
     def on_deselect_category(self, widget):
         selection = self.selected_categories.get_selection()
-        nb_selected, selected_iter = selection.get_selected()
+        _, selected_iter = selection.get_selected()
 
         if selected_iter is not None:
             model_available = self.available_categories.get_model()
@@ -253,7 +248,7 @@ class ContentsPage(AssistantPage):
             model_selected = self.selected_categories.get_model()
 
             for row in model_selected:
-                selected_categories.append(row[0].decode('utf-8'))
+                selected_categories.append(row[0])
 
             return selected_categories
 
@@ -275,7 +270,6 @@ class ContentsPage(AssistantPage):
         self.assistant.set_page_complete(self.assistant.page3, correct)
 
 
-
 class SummaryPage(AssistantPage):
     def __init__(self, *args, **kwargs):
         AssistantPage.__init__(self, *args, **kwargs)
@@ -288,11 +282,11 @@ class SummaryPage(AssistantPage):
         self.clear()
 
     def add_setting(self, setting, value):
-        label = gtk.Label()
+        label = Gtk.Label()
         label.set_markup('<b>%s:</b> %s' % (setting, value))
         label.set_alignment(0.0, 0.5)
         label.show()
-        self.pack_start(label, False)
+        self.pack_start(label, False, False, 0)
         self.settings.append(label)
 
     def clear(self):
@@ -301,7 +295,6 @@ class SummaryPage(AssistantPage):
         self.settings = []
 
 
-
 class ExportAssistant(Assistant):
     def __init__(self, *args, **kwargs):
         Assistant.__init__(self, *args, **kwargs)
@@ -311,9 +304,10 @@ class ExportAssistant(Assistant):
 
         self.set_title(_('Export Assistant'))
 
-        texts = [_('Welcome to the Export Assistant.'),
-                _('This wizard will help you to export your journal to various formats.'),
-                _('You can select the days you want to export and where the output will be saved.')]
+        texts = [
+            _('Welcome to the Export Assistant.'),
+            _('This wizard will help you to export your journal to various formats.'),
+            _('You can select the days you want to export and where the output will be saved.')]
         text = '\n'.join(texts)
         self._add_intro_page(text)
 
@@ -345,7 +339,7 @@ class ExportAssistant(Assistant):
         self.page5 = SummaryPage()
         self.append_page(self.page5)
         self.set_page_title(self.page5, _('Summary') + ' (5/5)')
-        self.set_page_type(self.page5, gtk.ASSISTANT_PAGE_CONFIRM)
+        self.set_page_type(self.page5, Gtk.AssistantPageType.CONFIRM)
         self.set_page_complete(self.page5, True)
 
         self.exporter = None
@@ -404,7 +398,9 @@ class ExportAssistant(Assistant):
                 start_date, end_date = self.page2.get_date_range()
                 if start_date == end_date:
                     self.page5.add_setting(_('Date'), start_date)
-                    self.page5.add_setting(_('Export selected text only'), self.yes_no(self.export_selected_text))
+                    self.page5.add_setting(
+                        _('Export selected text only'),
+                        self.yes_no(self.export_selected_text))
                 else:
                     self.page5.add_setting(_('Start date'), start_date)
                     self.page5.add_setting(_('End date'), end_date)
@@ -469,15 +465,13 @@ class ExportAssistant(Assistant):
         filesystem.write_file(self.path, export_string)
         self.journal.show_message(_('Content exported to %s') % self.path)
 
-
     def export_pdf(self):
         logging.info('Exporting to PDF')
         browser.print_pdf(self.get_export_string('xhtml'), self.path)
         self.journal.show_message(_('Content exported to %s') % self.path)
 
 
-
-class Exporter(object):
+class Exporter:
     NAME = 'Which format do we use?'
     # Short description of how we export
     DESCRIPTION = ''
@@ -492,7 +486,8 @@ class Exporter(object):
             try:
                 __import__(module)
             except ImportError:
-                logging.info('"%s" could not be imported. '
+                logging.info(
+                    '"%s" could not be imported. '
                     'You will not be able to import %s' % (module, cls.NAME))
                 # Importer cannot be used
                 return False
@@ -508,7 +503,6 @@ class Exporter(object):
         '''
         return True
 
-
     def export(self):
         '''
         This function has to be implemented by all subclasses
@@ -518,65 +512,50 @@ class Exporter(object):
 
     @property
     def DEFAULTPATH(self):
-        return os.path.join(os.path.expanduser('~'),
+        return os.path.join(
+            os.path.expanduser('~'),
             'RedNotebook-Export_%s.%s' % (datetime.date.today(), self.EXTENSION))
 
 
 class PlainTextExporter(Exporter):
     NAME = _('Plain Text')
-    #DESCRIPTION = 'Export journal to a plain textfile'
     EXTENSION = 'txt'
     FORMAT = 'txt'
 
 
 class HtmlExporter(Exporter):
     NAME = 'HTML'
-    #DESCRIPTION = 'Export journal to HTML'
     EXTENSION = 'html'
     FORMAT = 'xhtml'
 
 
 class LatexExporter(Exporter):
     NAME = 'Latex'
-    #DESCRIPTION = 'Create a tex file'
     EXTENSION = 'tex'
     FORMAT = 'tex'
 
 
 class PdfExporter(Exporter):
     NAME = 'PDF'
-    #DESCRIPTION = 'Create a PDF file'
     EXTENSION = 'pdf'
     FORMAT = 'pdf'
 
+    @classmethod
+    def is_available(cls):
+        return bool(browser.WebKit2)
+
     @property
     def DESCRIPTION(self):
         if self.is_available():
             return ''
+        elif filesystem.IS_WIN:
+            return _('(export to HTML, open in browser and print to PDF file)')
         else:
-            return '(' + _('requires pywebkitgtk') + ')'
-
-    @classmethod
-    def is_available(cls):
-        return browser.can_print_pdf()
-
-
+            return _('(requires WebKit2Gtk+ 4.0)')
 
 
 def get_exporters():
     exporters = [PlainTextExporter, HtmlExporter, LatexExporter, PdfExporter]
 
-    # Instantiate importers
-    exporters = map(lambda exporter: exporter(), exporters)
-    return exporters
-
-
-
-if __name__ == '__main__':
-    '''
-    Run some tests
-    '''
-    assistant = ExportAssistant(Journal())
-    assistant.set_position(gtk.WIN_POS_CENTER)
-    assistant.run()
-    gtk.main()
+    # Instantiate exporters
+    return [exporter() for exporter in exporters]
