@@ -17,61 +17,77 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 # -----------------------------------------------------------------------
 
-import sys
-import datetime
-import os
-import time
-import itertools
-import logging
-import locale
 from collections import defaultdict
+import datetime
+import itertools
+import locale
+import logging
+import os
+import sys
+import time
 
 
-# Use basic stdout logging before we can initialize logging correctly
-logging.basicConfig(level=logging.DEBUG,
-                    format='%(levelname)-8s %(message)s',
-                    stream=sys.stdout)
+# Use basic stdout logging before we can initialize logging correctly.
+logging.basicConfig(
+    level=logging.DEBUG,
+    format='%(levelname)-8s %(message)s',
+    stream=sys.stdout)
 
+try:
+    import gi
+except ImportError as err:
+    logging.error(
+        'pygobject could not be imported: "{}". Please install it (python3-gi).'.format(err))
+    sys.exit(1)
+
+gi.require_version("Gtk", "3.0")
 
-# Allow importing from rednotebook package
 if hasattr(sys, "frozen"):
-    from rednotebook.util import filesystem
-    assert filesystem  # silence pyflakes
+    base_dir = sys._MEIPASS
 else:
-    from util import filesystem
+    app_dir = os.path.dirname(os.path.abspath(__file__))
+    base_dir = os.path.dirname(app_dir)
 
-# Add base directory to sys.path
-base_dir = os.path.abspath(os.path.join(filesystem.app_dir, '../'))
+print('Adding {} to sys.path'.format(base_dir))
 sys.path.insert(0, base_dir)
 
+from rednotebook.util import filesystem
 
-## ---------------------- Enable i18n -------------------------------
+
+# ---------------------- Enable i18n -------------------------------
 
 from rednotebook.external import elibintl
 
-# We need to translate 3 different types of strings:
-# * sourcecode strings
-# * gtkbuilder strings
-# * gtk stock names
+LOCALE_PATH = filesystem.locale_dir
 
-LOCALE_PATH = filesystem.get_utf8_path(filesystem.locale_dir)
-
-# the name of the gettext domain.
 GETTEXT_DOMAIN = 'rednotebook'
 
-# Register _() as a global translation function and set up the translation
-try:
-    elibintl.install(GETTEXT_DOMAIN, LOCALE_PATH)
-except locale.Error, err:
-    # unsupported locale setting
-    logging.error('Locale could not be set: "%s"' % err)
-    logging.error('Probably you have to install the appropriate language packs')
-    # Make the _() function available even if gettext is not working.
-    import __builtin__
-    if not hasattr(__builtin__, '_'):
-        __builtin__.__dict__['_'] = lambda s: s
+"""
+On Windows, translations currently only work with a hack. We need to
+translate
 
-## ------------------- end Enable i18n -------------------------------
+  1) Strings in source code.
+  2) GTK stock items.
+  3) Strings in the Glade file.
+
+1) works fine. For 3) we need a workaround in main_window.py. If we
+initialize libintl with elibintl, the translations for 2) stop working.
+Without the workaround we need to initalize libintl to avoid UTF-8
+encoding errors.
+
+When the problem is fixed upstream, we can pass libintl='libintl-8.dll'
+to elibintl. See also
+
+https://bugzilla.gnome.org/show_bug.cgi?id=574520
+https://bugzilla.gnome.org/show_bug.cgi?id=753991
+https://github.com/tobias47n9e/pygobject-locale
+https://sourceforge.net/p/pygobjectwin32/tickets/27/
+
+"""
+
+elibintl.install(GETTEXT_DOMAIN, LOCALE_PATH, libintl=None)
+
+# ------------------- end Enable i18n -------------------------------
 
 
 from rednotebook.util import utils
@@ -83,7 +99,8 @@ from rednotebook import data
 
 args = info.get_commandline_parser().parse_args()
 
-## ---------------------- Enable logging -------------------------------
+# ---------------------- Enable logging -------------------------------
+
 
 def setup_logging(log_file):
     file_logging_stream = open(log_file, 'w')
@@ -126,35 +143,22 @@ default_config = configuration.Config(default_config_file)
 dirs = filesystem.Filenames(default_config)
 setup_logging(dirs.log_file)
 
-## ------------------ end Enable logging -------------------------------
+# ------------------ end Enable logging -------------------------------
 
 logging.info('System encoding: %s' % filesystem.ENCODING)
 logging.info('Language code: %s' % filesystem.LANGUAGE)
 
 try:
-    import pygtk
-    if not sys.platform == 'win32':
-        pygtk.require("2.0")
-except ImportError:
-    logging.error('pygtk not found. Please install PyGTK (python-gtk2)')
-    sys.exit(1)
-
-try:
-    import gtk
-    import gobject
-    # Some notes on threads_init:
-    # only gtk.gdk.threads_init(): pdf export works, but gui hangs afterwards
-    # only gobject.threads_init(): pdf export works, gui works
-    # both: pdf export works, gui hangs afterwards
-    gobject.threads_init()  # only initializes threading in the glib/gobject module
-    #gtk.gdk.threads_init()  # also initializes the gdk threads
-except (ImportError, AssertionError), e:
+    from gi.repository import Gtk
+    from gi.repository import GObject
+except (ImportError, AssertionError) as e:
     logging.error(e)
-    logging.error('gtk not found. Please install PyGTK (python-gtk2)')
+    logging.error('GTK+ not found. Please install it (gir1.2-gtk-3.0).')
     sys.exit(1)
 
+GObject.threads_init()
+
 
-# This version of import is needed for win32 to work
 from rednotebook.util import dates
 from rednotebook import backup
 
@@ -193,8 +197,6 @@ class Journal:
         logging.info('RedNotebook version: %s' % info.version)
         logging.info(filesystem.get_platform_info())
 
-        utils.set_environment_variables(self.config)
-
         self.actual_date = self.get_start_date()
 
         # Let components check if the MainWindow has been created
@@ -211,15 +213,14 @@ class Journal:
         self.open_journal(journal_path)
 
         self.archiver = backup.Archiver(self)
-        #self.archiver.check_last_backup_date()
+        GObject.idle_add(self.archiver.check_last_backup_date)
 
         # Check for a new version
         if self.config.read('checkForNewVersion') == 1:
             utils.check_new_version(self, info.version, startup=True)
 
         # Automatically save the content after a period of time
-        gobject.timeout_add_seconds(600, self.save_to_disk)
-
+        GObject.timeout_add_seconds(600, self.save_to_disk)
 
     def get_journal_path(self):
         '''
@@ -253,7 +254,7 @@ class Journal:
 
         logging.error('The path "%s" is not a valid journal directory. '
                       'Execute "rednotebook -h" for instructions' % path_arg)
-        sys.exit(1)
+        sys.exit(2)
 
     def get_start_date(self):
         '''
@@ -281,7 +282,7 @@ class Journal:
             # Informs the logging system to perform an orderly shutdown by
             # flushing and closing all handlers.
             logging.shutdown()
-            gtk.main_quit()
+            Gtk.main_quit()
 
     def convert(self, text, target, headers=None, options=None):
         options = options or {}
@@ -322,7 +323,6 @@ class Journal:
         # tell gobject to keep saving the content in regular intervals
         return True
 
-
     def open_journal(self, data_dir):
         if not os.path.exists(data_dir):
             logging.warning('The dir %s does not exist. Select a different dir.'
@@ -355,7 +355,7 @@ class Journal:
 
         self.frame.categories_tree_view.categories = self.categories
         # Add auto-completion for tag search
-        self.frame.search_box.set_entries([u'#%s' % self.normalize_tag(tag)
+        self.frame.search_box.set_entries(['#%s' % self.normalize_tag(tag)
                                            for tag in self.categories])
 
         self.title = filesystem.get_journal_title(data_dir)
@@ -391,7 +391,6 @@ class Journal:
 
         return self.months[year_and_month]
 
-
     def save_old_day(self):
         '''Order is important'''
         old_content = self.day.content
@@ -404,19 +403,17 @@ class Journal:
 
         self.frame.calendar.set_day_edited(self.date.day, not self.day.empty)
 
-
     def load_day(self, new_date):
         old_date = self.date
         self.date = new_date
 
-        if not Month.same_month(new_date, old_date) or self.month is None:
+        if not dates.same_month(new_date, old_date) or self.month is None:
             self.month = self.get_month(self.date)
 
         self.frame.set_date(self.month, self.date, self.day)
 
         self.set_frame_title()
 
-
     def merge_days(self, days):
         '''
         Method used by importers
@@ -429,12 +426,10 @@ class Journal:
             old_day.merge(new_day)
             month.edited = True
 
-
     @property
     def day(self):
         return self.month.get_day(self.date.day)
 
-
     def change_date(self, new_date):
         if new_date < datetime.date(1900, 1, 1):
             self.show_message('Only dates after 1900 are supported.',
@@ -447,7 +442,6 @@ class Journal:
         self.save_old_day()
         self.load_day(new_date)
 
-
     def go_to_next_day(self):
         next_date = self.date + dates.one_day
         following_edited_days = self.get_days_in_date_range(start_date=next_date)
@@ -455,7 +449,6 @@ class Journal:
             next_date = following_edited_days[0].date
         self.change_date(next_date)
 
-
     def go_to_prev_day(self):
         prev_date = self.date - dates.one_day
         previous_edited_days = self.get_days_in_date_range(end_date=prev_date)
@@ -463,39 +456,34 @@ class Journal:
             prev_date = previous_edited_days[-1].date
         self.change_date(prev_date)
 
-
     def show_message(self, msg, title=None, error=False):
         if error and not title:
             title = _('Error')
 
         if error:
-            msg_type = gtk.MESSAGE_ERROR
+            msg_type = Gtk.MessageType.ERROR
             log_level = logging.ERROR
         else:
-            msg_type = gtk.MESSAGE_INFO
+            msg_type = Gtk.MessageType.INFO
             log_level = logging.INFO
 
         self.frame.show_message(title, msg, msg_type)
         logging.log(log_level, '%s. %s' % (title, msg) if title else msg)
 
-
     @property
     def categories(self):
         return list(sorted(set(itertools.chain.from_iterable(
-            day.categories for day in self.days)), cmp=locale.strcoll))
-
+            day.categories for day in self.days)), key=locale.strxfrm))
 
     def normalize_tag(self, tag):
         return tag.replace(' ', '_').lower()
 
-
     def get_entries(self, category):
         entries = set()
         for day in self.days:
             entries |= set(day.get_entries(category))
         return sorted(entries)
 
-
     def search(self, text, tags):
         days = self.get_days_with_tags(tags)
         results = []
@@ -503,7 +491,6 @@ class Journal:
             results.append(day.search(text, tags))
         return results
 
-
     def get_days_with_tags(self, tags):
         if not tags:
             return self.days
@@ -514,13 +501,10 @@ class Journal:
                 days.append(day)
         return days
 
-
     def get_word_count_dict(self):
-        '''
-        Returns a dictionary mapping the words to their number of appearance
-        '''
-        # TODO: Use collections.Counter in Python2.7
-        # TODO: Check if concatenating all text and using a regex is faster.
+        """
+        Return a dictionary mapping the words to their number of appearance.
+        """
         word_dict = defaultdict(int)
         for day in self.days:
             words = day.get_words()
@@ -528,7 +512,6 @@ class Journal:
                 word_dict[word.lower()] += 1
         return word_dict
 
-
     @property
     def days(self):
         '''
@@ -540,17 +523,14 @@ class Journal:
 
         days = []
         for month in self.months.values():
-            days_in_month = month.days.values()
-
-            # Filter out days without content
-            days_in_month = [day for day in days_in_month if not day.empty]
+            # Filter out days without content.
+            days_in_month = [day for day in month.days.values() if not day.empty]
             days.extend(days_in_month)
 
         # Sort days
         days = sorted(days, key=lambda day: day.date)
         return days
 
-
     def get_days_in_date_range(self, start_date=None, end_date=None):
         if not start_date:
             start_date = datetime.date.min
@@ -570,7 +550,6 @@ class Journal:
                 break
         return days_in_date_range
 
-
     def add_instruction_content(self):
         self.change_date(datetime.date.today())
         current_date = self.date
@@ -585,7 +564,6 @@ class Journal:
         self.change_date(current_date)
 
 
-
 def main():
     start_time = time.time()
     journal = Journal()
@@ -595,10 +573,8 @@ def main():
 
     try:
         logging.debug('Trying to enter the gtk main loop')
-        gtk.main()
+        Gtk.main()
     except KeyboardInterrupt:
-        # 'Interrupt'
-        #journal.save_to_disk()
         sys.exit()
 
 
