@@ -21,6 +21,7 @@ import codecs
 import logging
 import os
 import re
+import shutil
 import stat
 import sys
 
@@ -28,14 +29,12 @@ import sys
 try:
     import yaml
 except ImportError:
-    logging.error('PyYAML not found. Please install python-yaml or PyYAML')
+    logging.error('PyYAML not found. Please install it (python3-yaml).')
     sys.exit(1)
 
-# The presence of the yaml module has been checked
 try:
     from yaml import CLoader as Loader
     from yaml import CSafeDumper as Dumper
-    assert Loader and Dumper  # silence pyflakes
 except ImportError:
     from yaml import Loader, Dumper
     logging.info('Using pyyaml for loading and dumping')
@@ -43,6 +42,10 @@ except ImportError:
 from rednotebook.data import Month
 
 
+def format_year_and_month(year, month):
+    return '%04d-%02d' % (year, month)
+
+
 def get_journal_files(data_dir):
     # Format: 2010-05.txt
     date_exp = re.compile(r'(\d{4})-(\d{2})\.txt$')
@@ -66,21 +69,21 @@ def _load_month_from_disk(path, year_number, month_number):
     If an error occurs, return None
     '''
     try:
-        # Try to read the contents of the file
+        # Try to read the contents of the file.
         with codecs.open(path, 'rb', encoding='utf-8') as month_file:
             logging.debug('Loading file "%s"' % path)
             month_contents = yaml.load(month_file, Loader=Loader)
-            month = Month(year_number, month_number, month_contents)
+            month = Month(year_number, month_number, month_contents, os.path.getmtime(path))
             return month
-    except yaml.YAMLError, exc:
+    except yaml.YAMLError as exc:
         logging.error('Error in file %s:\n%s' % (path, exc))
     except IOError:
-        #If that fails, there is nothing to load, so just display an error message
+        # If that fails, there is nothing to load, so just display an error message.
         logging.error('Error: The file %s could not be read' % path)
     except Exception:
         logging.error('An error occured while reading %s:' % path)
         raise
-    # If we continued here, the possibly corrupted file would be overwritten
+    # If we continued here, the possibly corrupted file would be overwritten.
     sys.exit(1)
 
 
@@ -95,46 +98,73 @@ def load_all_months_from_disk(data_dir):
     for path, year_number, month_number in get_journal_files(data_dir):
         month = _load_month_from_disk(path, year_number, month_number)
         if month:
-            months['%d-%02d' % (year_number, month_number)] = month
+            months[format_year_and_month(year_number, month_number)] = month
 
     logging.debug('Finished loading files in dir "%s"' % data_dir)
     return months
 
 
-def save_months_to_disk(months, dir, frame, exit_imminent=False, changing_journal=False, saveas=False):
+def _save_month_to_disk(month, journal_dir):
+    """
+    When overwriting 2014-12.txt:
+        write new content to 2014-12.new.txt
+        cp 2014-12.txt 2014-12.old.txt
+        mv 2014-12.new.txt 2014-12.txt
+        rm 2014-12.old.txt
+    """
+    content = {}
+    for day_number, day in month.days.items():
+        if not day.empty:
+            content[day_number] = day.content
+
+    def get_filename(infix):
+        year_and_month = format_year_and_month(month.year_number, month.month_number)
+        return os.path.join(journal_dir, '%s%s.txt' % (year_and_month, infix))
+
+    old = get_filename('.old')
+    new = get_filename('.new')
+    filename = get_filename('')
+
+    # Do not save empty month files.
+    if not content and not os.path.exists(filename):
+        return False
+
+    with codecs.open(new, 'wb', encoding='utf-8') as f:
+        # Write readable unicode and no Python directives.
+        yaml.dump(content, f, Dumper=Dumper, allow_unicode=True)
+
+    if os.path.exists(filename):
+        mtime = os.path.getmtime(filename)
+        if mtime != month.mtime:
+            conflict = get_filename('.CONFLICT_BACKUP' + str(mtime))
+            logging.debug('Last edit time of %s conflicts with edit time at file load\n'
+                          '--> Backing up to %s' % (filename, conflict))
+            shutil.copy2(filename, conflict)
+        shutil.copy2(filename, old)
+    shutil.move(new, filename)
+    if os.path.exists(old):
+        os.remove(old)
+
+    try:
+        # Make file readable and writable only by the owner.
+        os.chmod(filename, stat.S_IRUSR | stat.S_IWUSR)
+    except OSError:
+        pass
+
+    month.edited = False
+    month.mtime = os.path.getmtime(filename)
+    logging.info('Wrote file %s' % filename)
+    return True
+
+
+def save_months_to_disk(months, journal_dir, exit_imminent=False, saveas=False):
     '''
-    Do the actual saving and return if something has been saved
+    Update the journal on disk and return if something had to be written.
     '''
     something_saved = False
     for year_and_month, month in months.items():
-        # We always need to save everything when we are "saving as"
+        # We always need to save everything when we are "saving as".
         if month.edited or saveas:
-            something_saved = True
-            month_file_string = os.path.join(dir, year_and_month + '.txt')
-            month_content = {}
-            for day_number, day in month.days.iteritems():
-                # do not add empty days
-                if not day.empty:
-                    month_content[day_number] = day.content
-
-            # Do not save empty month files
-            if not month_content and not os.path.exists(month_file_string):
-                continue
-
-            with codecs.open(month_file_string, 'wb', encoding='utf-8') as month_file:
-                try:
-                    # This version produces readable unicode and no python directives
-                    yaml.dump(month_content, month_file, Dumper=Dumper, allow_unicode=True)
-                    month.edited = False
-                    logging.debug('Wrote file %s' % month_file_string)
-                except OSError:
-                    frame.show_save_error_dialog(exit_imminent)
-                except IOError:
-                    frame.show_save_error_dialog(exit_imminent)
-            try:
-                # Make file readable and writable only by the owner.
-                os.chmod(month_file_string, stat.S_IRUSR | stat.S_IWUSR)
-            except OSError:
-                pass
+            something_saved |= _save_month_to_disk(month, journal_dir)
 
     return something_saved
