@@ -34,12 +34,6 @@ REMOTE_PROTOCOLS = ['http', 'ftp', 'irc']
 IS_WIN = sys.platform.startswith('win')
 
 
-def get_unicode_path(path):
-    return unicode(path, ENCODING)
-
-def get_utf8_path(path):
-    return path.encode('UTF-8')
-
 def has_system_tray():
     return IS_WIN  # A smarter detection is needed here ;)
 
@@ -52,7 +46,6 @@ if main_is_frozen():
     app_dir = sys._MEIPASS  # os.path.dirname(sys.executable)
 else:
     app_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
-app_dir = get_unicode_path(app_dir)
 
 if IS_WIN:
     locale_dir = os.path.join(app_dir, 'share', 'locale')
@@ -63,7 +56,7 @@ image_dir = os.path.join(app_dir, 'images')
 frame_icon_dir = os.path.join(image_dir, 'rednotebook-icon')
 files_dir = os.path.join(app_dir, 'files')
 
-user_home_dir = get_unicode_path(os.path.expanduser('~'))
+user_home_dir = os.path.expanduser('~')
 
 
 class Filenames(dict):
@@ -73,7 +66,7 @@ class Filenames(dict):
     def __init__(self, config):
         for key, value in globals().items():
             # Exclude "get_main_dir()"
-            if key.lower().endswith('dir') and isinstance(value, basestring):
+            if key.lower().endswith('dir') and isinstance(value, str):
                 value = os.path.abspath(value)
                 self[key] = value
                 setattr(self, key, value)
@@ -94,7 +87,6 @@ class Filenames(dict):
 
         self.forbidden_dirs = [user_home_dir, self.journal_user_dir]
 
-
     def get_user_dir(self, config):
         custom = config.read('userDir')
 
@@ -113,11 +105,9 @@ class Filenames(dict):
 
         return user_dir
 
-
     def is_valid_journal_path(self, path):
         return os.path.isdir(path) and os.path.abspath(path) not in self.forbidden_dirs
 
-
     def __getattribute__(self, attr):
         user_paths = dict((
             ('template_dir', 'templates'),
@@ -133,48 +123,19 @@ class Filenames(dict):
         return dict.__getattribute__(self, attr)
 
 
-
 def read_file(filename):
-    '''Tries to read a given file
-
-    Returns None if an error is encountered
-    '''
-    encodings = ['utf-8']
+    """Try to read a given file.
 
+    Return empty string if an error is encountered.
+    """
     try:
-        import chardet
-        assert chardet  # silence pyflakes
-    except ImportError:
-        logging.info("chardet not found. 'utf-8' encoding will be assumed")
-        chardet = None
-
-    if False and chardet:
-        with open(filename, 'rb') as file:
-            content = file.read()
-        guess = chardet.detect(content)
-        logging.info('Chardet guesses %s for %s' % (guess, filename))
-        encoding = guess.get('encoding')
-
-        # chardet makes errors here sometimes
-        if encoding in ['MacCyrillic', 'ISO-8859-7']:
-            encoding = 'ISO-8859-2'
-
-        if encoding:
-            encodings.insert(0, encoding)
-
-    # Only check the first encoding
-    for encoding in encodings[:1]:
-        try:
-            # codecs.open returns a file object that can write unicode objects
-            # and whose read() method also returns unicode objects
-            # Internally we want to have unicode only
-            with codecs.open(filename, 'rb', encoding=encoding, errors='replace') as file:
-                data = file.read()
-                return data
-        except ValueError, err:
-            logging.info(err)
-        except Exception, e:
-            logging.error(e)
+        with codecs.open(filename, 'rb', encoding='utf-8', errors='replace') as file:
+            data = file.read()
+            return data
+    except ValueError as err:
+        logging.info(err)
+    except Exception as err:
+        logging.error(err)
     return ''
 
 
@@ -183,7 +144,7 @@ def write_file(filename, content):
     try:
         with codecs.open(filename, 'wb', errors='replace', encoding='utf-8') as file:
             file.write(content)
-    except IOError, e:
+    except IOError as e:
         logging.error('Error while writing to "%s": %s' % (filename, e))
 
 
@@ -191,43 +152,42 @@ def make_directory(dir):
     if not os.path.isdir(dir):
         os.makedirs(dir)
 
+
 def make_directories(dirs):
     for dir in dirs:
         make_directory(dir)
 
+
 def make_file(file, content=''):
     if not os.path.isfile(file):
         write_file(file, content)
 
+
 def make_files(file_content_pairs):
     for file, content in file_content_pairs:
         make_file(file, content)
 
+
 def make_file_with_dir(file, content):
     dir = os.path.dirname(file)
     make_directory(dir)
     make_file(file, content)
 
+
 def get_relative_path(from_dir, to_dir):
     '''
     Try getting the relative path from from_dir to to_dir
-    The relpath method is only available in python >= 2.6
-    if we run python <= 2.5, return the absolute path to to_dir
     '''
-    if getattr(os.path, 'relpath', None):
-        # If the data is saved on two different windows partitions,
-        # return absolute path to to_dir
-        drive1, tail = os.path.splitdrive(from_dir)
-        drive2, tail = os.path.splitdrive(to_dir)
-
-        # drive1 and drive2 are always empty strings on Unix
-        if not drive1.upper() == drive2.upper():
-            return to_dir
-
-        return os.path.relpath(to_dir, from_dir)
-    else:
+    # If the data is saved on two different windows partitions,
+    # return absolute path to to_dir.
+    # drive1 and drive2 are always empty strings on Unix.
+    drive1, _ = os.path.splitdrive(from_dir)
+    drive2, _ = os.path.splitdrive(to_dir)
+    if drive1.upper() != drive2.upper():
         return to_dir
 
+    return os.path.relpath(to_dir, from_dir)
+
 
 def get_journal_title(dir):
     '''
@@ -243,22 +203,22 @@ def get_journal_title(dir):
 
 
 def get_platform_info():
-    import gtk
+    from gi.repository import GObject
+    from gi.repository import Gtk
     import yaml
 
-    functions = [platform.machine, platform.platform, platform.processor,
-                 platform.python_version, platform.release, platform.system]
+    functions = [
+        platform.machine, platform.platform, platform.processor,
+        platform.python_version, platform.release, platform.system
+    ]
     names_values = [(func.__name__, func()) for func in functions]
 
-    lib_values = [('GTK version', gtk, 'gtk_version'),
-                  ('PyGTK version', gtk, 'pygtk_version'),
-                  ('Yaml version', yaml, '__version__')]
-
-    for name, object, value in lib_values:
-        try:
-            names_values.append((name, getattr(object, value)))
-        except AttributeError:
-            logging.info('%s could not be determined' % name)
+    names_values.extend([
+        ('GTK', (Gtk.get_major_version(), Gtk.get_minor_version(), Gtk.get_micro_version())),
+        ('Glib', GObject.glib_version),
+        ('PyGObject', GObject.pygobject_version),
+        ('YAML', yaml.__version__),
+        ])
 
     vals = ['%s: %s' % (name, val) for name, val in names_values]
     return 'System info: ' + ', '.join(vals)
@@ -304,8 +264,8 @@ def open_url_in_browser(url):
 
 
 def unquote_url(url):
-    import urllib
-    return urllib.unquote(url).decode('utf-8')
+    import urllib.parse
+    return urllib.parse.unquote(url)
 
 
 def _open_url_with_call(url, prog):
@@ -317,6 +277,7 @@ def _open_url_with_call(url, prog):
         # If everything failed, try the webbrowser
         open_url_in_browser(url)
 
+
 def open_url(url):
     '''
     Opens a file with the platform's preferred method
@@ -329,7 +290,7 @@ def open_url(url):
     if IS_WIN:
         try:
             url = unquote_url(url)
-            if url.startswith(u'file:') or os.path.exists(url):
+            if url.startswith('file:') or os.path.exists(url):
                 url = get_local_url(url)
             logging.info('Trying to open %s with "os.startfile"' % url)
             # os.startfile is only available on windows
@@ -340,12 +301,3 @@ def open_url(url):
         _open_url_with_call(url, 'open')
     else:
         _open_url_with_call(url, 'xdg-open')
-
-
-
-if __name__ == '__main__':
-    dirs = ['/home/my journal', '/my journal/', r'C:\\Dok u E\journal',
-            '/home/name/journal', '/']
-    for dir in dirs:
-        title = get_journal_title(dir)
-        print '%s -> %s' % (dir, title)
